import requests

def get_project_sheets(auth_client, project_uuid):
    """
    Get the list of project sheets.

    :param project_uuid: The UUID of the project.
    :param token: The bearer token for authorization.
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/project/{project_uuid}/sheet/list"
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Accept': 'application/json'
    }

    response = requests.get(url, headers=headers, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(data)

    return data

def get_sheet_history(auth_client, project_uuid, sheet_uuid):
    """
    Get a sheet's version history.

    :param project_uuid: The UUID of the project.
    :param sheet_uuid: The UUID of the sheet.
    :param token: The bearer token for authorization.
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/project/{project_uuid}/sheet/{sheet_uuid}/history"
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Accept': 'application/json'
    }

    response = requests.get(url, headers=headers, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(data)

    return data

def get_sheet_filter_options(auth_client, project_uuid):
    """
    Get the list of sheet filter options.

    :param project_uuid: The UUID of the project.
    :param token: The bearer token for authorization.
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/project/{project_uuid}/sheet/field-variants"
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Accept': 'application/json'
    }

    response = requests.get(url, headers=headers, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(data)

    return data




import requests
import uuid
from datetime import datetime
from typing import Optional, List, Dict, Any


def add_comment(
    auth_client: Any,
    project_uuid: str,
    issue_uuid: str,
    comment_type: str,
    reporter: str,
    comment_text: Optional[str] = None,
    file_content: Optional[bytes] = None,
    old_watchers: Optional[List[str]] = None,
    new_watchers: Optional[List[str]] = None,
) -> Dict[str, Any]:
    """
    Adds a comment to an issue on Revizto.

    Parameters:
        auth_client (Any): Authenticated client instance.
        project_uuid (str): UUID of the project.
        issue_uuid (str): UUID of the issue.
        comment_type (str): Type of the comment (text, file, markup, diff).
        reporter (str): Email of the reporter.
        comment_text (Optional[str]): Text of the comment (for text comments).
        file_content (Optional[bytes]): Binary content of the file (for file and markup comments).
        old_watchers (Optional[List[str]]): List of old watchers (for diff comments).
        new_watchers (Optional[List[str]]): List of new watchers (for diff comments).

    Returns:
        Dict[str, Any]: JSON response from the API.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/comment/add"
    headers = {
        "Authorization": f"Bearer {auth_client.access_token}",
        "Content-Type": "multipart/form-data",
    }

    comment_uuid = str(uuid.uuid4())
    created_timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    comments = [
        {
            "type": comment_type,
            "uuid": comment_uuid,
            "reporter": reporter,
            "created": created_timestamp,
            "rClashSync": False,
        }
    ]

    if comment_type == "text":
        comments[0]["text"] = comment_text
    elif comment_type in ["file", "markup"]:
        file_key = f"file_{comment_uuid}"
        comments[0]["file_key"] = file_key
    elif comment_type == "diff":
        comments[0]["diff"] = {"title": {"old": old_watchers, "new": new_watchers}}

    data = {
        "project_uuid": project_uuid,
        "issue_uuid": issue_uuid,
        "comments": comments,
    }

    files = {"file_content": file_content} if file_content else None

    try:
        response = requests.post(
            url,
            headers=headers,
            data=data,
            files=files,
            verify=auth_client.verification_bool,
            cert=auth_client.certificate,
        )
        response.raise_for_status()
        return response.json()
    except requests.exceptions.RequestException as e:
        raise RuntimeError(f"Failed to add comment: {e}")


def get_issue_comments(
    auth_client: Any, project_id: int, issue_uuid: str, date: str, page: int = 0
) -> Dict[str, Any]:
    """
    Gets issue comments added on the specified date or later.

    Parameters:
        auth_client (Any): Authenticated client instance.
        project_id (int): ID of the project.
        issue_uuid (str): UUID of the issue.
        date (str): Date in the format YYYY-MM-DD.
        page (int): Page number for pagination. Default is 0.

    Returns:
        Dict[str, Any]: JSON response from the API.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/issue/{issue_uuid}/comments/date"
    headers = {
        "Authorization": f"Bearer {auth_client.access_token}",
        "Content-Type": "application/json",
    }

    params = {"project_id": project_id, "date": date, "page": page}

    try:
        response = requests.get(
            url,
            headers=headers,
            params=params,
            verify=auth_client.verification_bool,
            cert=auth_client.certificate,
        )
        response.raise_for_status()
        return response.json()
    except requests.exceptions.RequestException as e:
        raise RuntimeError(f"Failed to get issue comments: {e}")



import requests

def get_license_projects(auth_client, license_uuid, avatars=None, notifications=None, page=0, screenshots=None, sorting=None, type=None):
    """
    Get the current user's projects in the specified license.

    :param license_uuid: The UUID of the license.
    :param token: The bearer token for authorization.
    :param avatars: Include avatars in the response (boolean).
    :param notifications: Include notifications in the response (boolean).
    :param page: The page number to retrieve (default is 0).
    :param screenshots: Include screenshots in the response (boolean).
    :param sorting: The sorting parameter (string).
    :param type: The type parameter (string).
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/project/list/{license_uuid}/paged"
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Content-Type': 'application/json'
    }
    
    params = {
        'page': page
    }
    
    if avatars is not None:
        params['avatars'] = avatars
    if notifications is not None:
        params['notifications'] = notifications
    if screenshots is not None:
        params['screenshots'] = screenshots
    if sorting is not None:
        params['sorting'] = sorting
    if type is not None:
        params['type'] = type

    response = requests.get(url, headers=headers, params=params, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(f"API Error: {data['message']}")

    return data

def get_project_members(auth_client, project_uuid):
    """
    Get the list of project members.

    :param project_uuid: The UUID of the project.
    :param token: The bearer token for authorization.
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/project/{project_uuid}/team"
    
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Accept': 'application/json'
    }

    response = requests.get(url, headers=headers, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(f"API Error: {data['message']}")

    return data

##### Giving some unkown error ####################
def invite_users_to_project(auth_client, project_uuid, invitations, role_id, operation_id=None):
    """
    Invite users to a project.

    :param project_uuid: The UUID of the project.
    :param token: The bearer token for authorization.
    :param invitations: A list of email addresses to invite.
    :param role_id: The ID of the project role to assign to new project members.
    :param operation_id: An optional unique string associated with notifications.
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/project/{project_uuid}/role/invite"
    
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Accept': 'application/json',
        'Content-Type': 'application/json'
    }
    
    payload = {
        'invitations': invitations,
        'roleId': role_id,
    }
    
    if operation_id is not None:
        payload['operationId'] = operation_id

    response = requests.post(url, headers=headers, json=payload, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(data)

    return data


def remove_users_from_project(auth_client, project_uuid, member_uuids, operation_id=None):
    """
    Remove users from a project.

    :param project_uuid: The UUID of the project.
    :param token: The bearer token for authorization.
    :param member_uuids: A list of UUIDs of project members to be removed.
    :param operation_id: An optional unique string associated with notifications.
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/project/{project_uuid}/role/bulk-delete"
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Accept': 'application/json',
        'Content-Type': 'application/json'
    }
    
    payload = {
        'memberUuids': member_uuids,
        'operationId': None
    }
    
    if operation_id is not None:
        payload['operationId'] = operation_id

    response = requests.post(url, headers=headers, json=payload, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(data)

    return data



import requests
import json
import uuid
import mimetypes
from typing import Optional, List, Dict, Any


def get_project_issues(
    auth_client: Any,
    project_uuid: str,
    page: int = 0,
    always_filters: List[Dict[str, Any]] = None,
    any_filters: List[Dict[str, Any]] = None,
    additional_fields: List[str] = None,
    report_sort: List[Dict[str, str]] = None,
    limit: int = 100,
    send_full_issue_data: bool = False,
    synchronized: str = None,
) -> List[Dict[str, Any]]:
    """
    Fetches a list of issues for a project, with support for filters, sorting, and pagination.

    Args:
        auth_client: Authenticated client instance.
        project_uuid (str): The UUID of the project.
        page (int) : The page number.
        always_filters (List[Dict[str, Any]], optional): Filters to always apply.
        any_filters (List[Dict[str, Any]], optional): Filters to optionally apply.
        additional_fields (List[str], optional): Additional fields to include in the response.
        report_sort (List[Dict[str, str]], optional): Sorting parameters.
        limit (int, optional): Number of issues per page. Default is 100.
        send_full_issue_data (bool, optional): Whether to fetch full issue data or only changed fields.
        synchronized (str, optional): Fetch issues created/updated after a specific time (YYYY-MM-DD HH:MM:SS).

    Returns:
        List[Dict[str, Any]]: A list of all fetched issues.
    """
    
    url = f"https://api.{auth_client.region}.revizto.com/v5/project/{project_uuid}/issue-filter/filter"
    headers = {
        "Authorization": f"Bearer {auth_client.access_token}",
        "Content-Type": "application/json",
    }


    params = {
        "limit": limit,
        "page": page,
        "sendFullIssueData": send_full_issue_data,
        "synchronized": synchronized,
    }

    if additional_fields:
        params["additionalFields[]"] = additional_fields

    if always_filters:
        params["alwaysFiltersDTO[]"] = always_filters

    if any_filters:
        params["anyFiltersDTO[]"] = any_filters

    if report_sort:
        params["reportSort[]"] = report_sort

    response = requests.get(
        url,
        headers=headers,
        params=params,
        verify=auth_client.verification_bool,
        cert=auth_client.certificate,
    )

    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()

    if data["result"] != 0:
        raise Exception(f"API Error: {data['message']}")

    return data


def get_deleted_issues(
    auth_client: Any,
    project_uuid: str,
    page: int = 0,
    limit: int = 100,
    additional_fields: Optional[List[str]] = None,
    always_filters_dto: Optional[List[Dict[str, Any]]] = None,
    any_filters_dto: Optional[List[Dict[str, Any]]] = None,
    report_sort: Optional[List[str]] = None,
    send_full_issue_data: bool = False,
    statuses: Optional[List[str]] = None,
    synchronized: Optional[str] = None,
) -> Dict[str, Any]:
    """
    Get a list of deleted issues with optional filtering and sorting for a specific page.

    :param auth_client: Authenticated client instance.
    :param project_uuid: The UUID of the project.
    :param page: The page number to retrieve (default is 0).
    :param limit: The number of issues per page (default is 100).
    :param additional_fields: A list of additional fields to include in the response.
    :param always_filters_dto: A list of filters that must always be applied.
    :param any_filters_dto: A list of filters where any can be applied.
    :param report_sort: A list of sorting parameters.
    :param send_full_issue_data: Whether to send full issue data (default is False).
    :param statuses: A list of statuses to filter by.
    :param synchronized: A synchronization filter.

    :return:
        A dictionary containing a list of deleted issues and the total number of pages.
    """

    url = f"https://api.{auth_client.region}.revizto.com/v5/project/{project_uuid}/issue-filter/filter_deleted"
    headers = {
        "Authorization": f"Bearer {auth_client.access_token}",
        "Accept": "application/json",
    }
    params = {"page": page, "limit": limit, "sendFullIssueData": send_full_issue_data}
    if additional_fields:
        params["additionalFields[]"] = additional_fields
    if always_filters_dto:
        params["alwaysFiltersDTO[]"] = always_filters_dto
    if any_filters_dto:
        params["anyFiltersDTO[]"] = any_filters_dto
    if report_sort:
        params["reportSort[]"] = report_sort
    if statuses:
        params["statuses[]"] = statuses
    if synchronized:
        params["synchronized"] = synchronized

    response = requests.get(
        url,
        headers=headers,
        params=params,
        verify=auth_client.verification_bool,
        cert=auth_client.certificate,
    )

    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()

    if data["result"] != 0:
        raise Exception(data)

    return data


def create_issue(
    auth_client: Any,
    preview_file_path: str,
    project_id: int,
    fields_json: Dict[str, Any],
    clash_test_uuid: Optional[str] = None,
    operation_id: Optional[str] = None,
) -> Dict[str, Any]:
    """
    Create a new issue in a project.

    :param auth_client: Authenticated client instance.
    :param preview_file_path: The path to the preview file.
    :param project_id: The project ID.
    :param fields_json: The fields for the issue in JSON format.
    :param clash_test_uuid: The clash test UUID (optional).
    :param operation_id: A unique string associated with notifications (optional).
    :return: The JSON response from the API.
    """

    # API endpoint for creating an issue
    url = f"https://api.{auth_client.region}.revizto.com/v5/issue/add"

    # Generate a unique UUID for the issue
    issue_uuid = str(uuid.uuid4())

    # Set up the headers with the bearer token
    headers = {
        "Authorization": f"Bearer {auth_client.access_token}",
        "Accept": "application/json",
    }

    # Read the preview file in binary mode
    with open(preview_file_path, "rb") as file:
        # Prepare the multipart form-data
        files = {
            "preview": (
                preview_file_path,
                file,
                mimetypes.guess_type(preview_file_path)[0],
            ),
            "fields": (None, json.dumps(fields_json), "application/json"),
            "uuid": (None, issue_uuid),
            "projectId": (None, str(project_id)),
        }

        if operation_id is not None:
            files["operationId"] = (None, operation_id)
        if clash_test_uuid is not None:
            files["clashTestUuid"] = (None, clash_test_uuid)

        # Make the POST request to the API endpoint
        response = requests.post(
            url,
            headers=headers,
            data=files,
            verify=auth_client.verification_bool,
            cert=auth_client.certificate,
        )

    # Check if the request was successful
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()

    if data["result"] != 0:
        raise Exception(data)

    return data



import requests

def get_stamp_templates(auth_client, project_uuid, page=0):
    """
    Get the stamp templates and stamp template categories available in the project.

    :param project_uuid: The UUID of the project.
    :param token: The bearer token for authorization.
    :param page: The page number (default is 0).
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/project/{project_uuid}/issue-preset/list"
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Accept': 'application/json'
    }
    params = {
        'page': page
    }

    response = requests.get(url, headers=headers, params=params, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(data)

    return data


import os
import requests
from datetime import datetime, timedelta
from dotenv import load_dotenv, set_key
from typing import Optional, Union, List, Dict, Any
from .licenses import (
    get_license_members,
    get_current_user_licenses,
    invite_users_to_license,
    assign_license_roles,
    remove_license_members,
)
from .comments import get_issue_comments, add_comment
from .projects import (
    get_license_projects,
    get_project_members,
    invite_users_to_project,
    remove_users_from_project,
)
from .issues import get_project_issues, get_deleted_issues, create_issue
from .project_roles import get_project_roles, assign_project_role
from .users import get_current_user_info
from .stamps import get_stamp_templates
from .reports import get_user_reports
from .sheets import get_project_sheets, get_sheet_history, get_sheet_filter_options


class TokenError(Exception):
    """Custom exception for token-related errors."""

    pass


class pyRevizto:
    def __init__(
        self,
        region: str,
        client_id: Optional[str] = None,
        redirect_uri: Optional[str] = None,
        state: Optional[str] = None,
        scope: Optional[str] = None,
        verification_bool: Union[bool, str] = True,
        certificate: Optional[str] = None,
        save_token: bool = False,
        env_path: str = ".env",
        token_validity: timedelta = timedelta(hours=0.5),
        refresh_token_validity: timedelta = timedelta(days=30),
    ):
        """
        Initialize the pyRevizto instance.

        :param region: The region for the Revizto API.
        :param client_id: The client ID for OAuth2.
        :param redirect_uri: The redirect URI for OAuth2.
        :param state: The state parameter for OAuth2.
        :param scope: The scope parameter for OAuth2.
        :param verification_bool: Whether to verify SSL certificates.
        :param certificate: Path to SSL certificate.
        :param save_token: Whether to save tokens to the environment file.
        :param env_path: Path to the environment file.
        :param token_validity: Validity duration for the access token.
        :param refresh_token_validity: Validity duration for the refresh token.
        """
        self.region = region
        self.verification_bool = verification_bool
        self.certificate = certificate
        self.save_token = save_token
        self.env_path = env_path
        self.token_validity = token_validity
        self.refresh_token_validity = refresh_token_validity
        self.client_id = client_id
        self.redirect_uri = redirect_uri
        self.state = state
        self.scope = scope
        self.token_url = f"https://api.{region}.revizto.com/v5/oauth2"
        self.access_token = None
        self.refresh_token = None
        self.token_expiration = None
        self.refresh_token_expiration = None
        if self.save_token:
            self._ensure_env_file_exists()
            load_dotenv(env_path)
            self._update_tokens_from_file()

    def get_tokens(self, access_code: str) -> Dict[str, str]:
        if self._is_token_valid():
            return {
                "access_token": self.access_token,
                "refresh_token": self.refresh_token,
            }
        elif self._is_refresh_token_valid():
            return self.get_refreshed_token()
        else:
            return self._request_new_tokens(access_code)

    def get_refreshed_token(self) -> Dict[str, str]:
        if self._is_token_valid():
            return {
                "access_token": self.access_token,
                "refresh_token": self.refresh_token,
            }
        elif self._is_refresh_token_valid():
            return self._request_refresh_token()
        else:
            raise TokenError("Refresh token is invalid or expired")

    def _request_new_tokens(self, access_code: str) -> Dict[str, str]:
        params = {
            "grant_type": "authorization_code",
            "code": access_code,
            "redirect_uri": self.redirect_uri,
            "client_id": self.client_id,
            "state": self.state,
            "scope": self.scope,
        }

        try:
            response = requests.post(
                self.token_url,
                data=params,
                verify=self.verification_bool,
                cert=self.certificate,
            )
            return self._handle_token_response(response)
        except requests.exceptions.RequestException as e:
            raise RuntimeError(f"Failed to request new tokens: {e}")

    def _request_refresh_token(self) -> Dict[str, str]:
        params = {
            "grant_type": "refresh_token",
            "refresh_token": self.refresh_token,
            "redirect_uri": self.redirect_uri,
            "client_id": self.client_id,
        }

        try:
            response = requests.post(
                self.token_url,
                data=params,
                verify=self.verification_bool,
                cert=self.certificate,
            )
            return self._handle_token_response(response)
        except requests.exceptions.RequestException as e:
            raise RuntimeError(f"Failed to refresh token: {e}")

    def _handle_token_response(self, response: requests.Response) -> Dict[str, str]:
        if response.status_code == 200:
            tokens = response.json()
            if tokens.get("token_type") == "Bearer":
                self.access_token = tokens.get("access_token")
                self.refresh_token = tokens.get("refresh_token")
                self.refresh_token_expiration = datetime.now()
                self.token_expiration = datetime.now()
                self._save_tokens()
                return tokens
            else:
                raise PermissionError(tokens)
        elif response.status_code == 400:
            raise ValueError(
                "Invalid request: Check the request parameters and try again."
            )
        elif response.status_code == 401:
            raise PermissionError("Unauthorized: Check your client ID and secret.")
        elif response.status_code == 403:
            raise PermissionError(
                "Forbidden: You do not have permission to access this resource."
            )
        elif response.status_code == 500:
            raise RuntimeError("Server error: Try again later.")
        else:
            response.raise_for_status()

    def _update_tokens_from_file(self) -> None:
        if os.getenv(f"{self.region}_ACCESS_TOKEN_TIMESTAMP"):
            self.access_token = os.getenv(f"{self.region}_ACCESS_TOKEN")
            self.refresh_token = os.getenv(f"{self.region}_REFRESH_TOKEN")
            self.token_expiration = datetime.fromisoformat(os.getenv(f"{self.region}_ACCESS_TOKEN_TIMESTAMP"))
            self.refresh_token_expiration = datetime.fromisoformat(os.getenv(f"{self.region}_REFRESH_TOKEN_TIMESTAMP"))

    def _save_tokens(self) -> None:
        if self.save_token:
            set_key(self.env_path, f"{self.region}_ACCESS_TOKEN", self.access_token)
            set_key(self.env_path, f"{self.region}_ACCESS_TOKEN_TIMESTAMP", self.token_expiration.isoformat())
            set_key(self.env_path, f"{self.region}_REFRESH_TOKEN", self.refresh_token)
            set_key(self.env_path, f"{self.region}_REFRESH_TOKEN_TIMESTAMP", self.refresh_token_expiration.isoformat())

    def _is_refresh_token_valid(self) -> bool:
        if self.refresh_token_expiration:
            return (
                datetime.now() <= self.refresh_token_expiration + self.refresh_token_validity
            )
        return False

    def _is_token_valid(self) -> bool:
        if self.token_expiration:
            return datetime.now() <= self.token_expiration + self.token_validity
        return False

    def _ensure_env_file_exists(self) -> None:
        if not os.path.exists(self.env_path):
            open(self.env_path, "a").close()


    def get_current_user_licenses(self) -> Any:
        return get_current_user_licenses(self)

    def get_license_members(self, license_id: str) -> Any:
        return get_license_members(self, license_id)

    def get_issue_comments(
        self, project_id: int, issue_uuid: str, date: str, page: int = 0
    ) -> Any:
        return get_issue_comments(self, project_id, issue_uuid, date, page)

    def get_license_projects(
        self,
        license_uuid: str,
        avatars: Optional[bool] = None,
        notifications: Optional[bool] = None,
        page: int = 0,
        screenshots: Optional[bool] = None,
        sorting: Optional[str] = None,
        type: Optional[str] = None,
    ) -> Any:
        return get_license_projects(
            self, license_uuid, avatars, notifications, page, screenshots, sorting, type
        )

    def get_project_issues(
        self,
        project_uuid: str,
        page: int = 0,
        filters: Optional[List[Dict[str, Any]]] = None,
        sort: Optional[List[str]] = None,
        additional_fields: Optional[List[str]] = None,
        limit: int = 100,
    ) -> Any:
        return get_project_issues(
            self, project_uuid, page, filters, sort, additional_fields, limit
        )

    def add_comment(
        self,
        project_uuid: str,
        issue_uuid: str,
        comment_type: str,
        reporter: str,
        comment_text: Optional[str] = None,
        file_content: Optional[bytes] = None,
        old_watchers: Optional[List[str]] = None,
        new_watchers: Optional[List[str]] = None,
    ) -> Any:
        return add_comment(
            self,
            project_uuid,
            issue_uuid,
            comment_type,
            reporter,
            comment_text,
            file_content,
            old_watchers,
            new_watchers,
        )

    def invite_users_to_license(
        self,
        license_uuid: str,
        invite_data: List[Dict[str, Any]],
        preserve_roles: Optional[bool] = None,
        make_guests: Optional[bool] = None,
        auth_method: Optional[str] = None,
        deactivate: Optional[bool] = None,
        operation_id: Optional[str] = None,
    ) -> Any:
        return invite_users_to_license(
            self,
            license_uuid,
            invite_data,
            preserve_roles,
            make_guests,
            auth_method,
            deactivate,
            operation_id,
        )

    def assign_license_roles(
        self,
        license_uuid: str,
        member_uuids: List[str],
        role: int,
        operation_id: Optional[str] = None,
    ) -> Any:
        return assign_license_roles(
            self, license_uuid, member_uuids, role, operation_id
        )

    def remove_license_members(
        self,
        license_uuid: str,
        member_uuids: List[str],
        message: Optional[str] = None,
        operation_id: Optional[str] = None,
    ) -> Any:
        return remove_license_members(
            self, license_uuid, member_uuids, message, operation_id
        )

    def get_project_members(self, project_uuid: str) -> Any:
        return get_project_members(self, project_uuid)

    def invite_users_to_project(
        self,
        project_uuid: str,
        invitations: List[str],
        role_id: int,
        operation_id: Optional[str] = None,
    ) -> Any:
        return invite_users_to_project(
            self, project_uuid, invitations, role_id, operation_id
        )

    def get_project_roles(self, license_uuid: str) -> Any:
        return get_project_roles(self, license_uuid)

    def remove_users_from_project(
        self,
        project_uuid: str,
        member_uuids: List[str],
        operation_id: Optional[str] = None,
    ) -> Any:
        return remove_users_from_project(self, project_uuid, member_uuids, operation_id)

    def assign_project_role(
        self,
        project_uuid: str,
        member_uuids: List[str],
        role_uuid: str,
        operation_id: Optional[str] = None,
    ) -> Any:
        return assign_project_role(
            self, project_uuid, member_uuids, role_uuid, operation_id
        )

    def get_current_user_info(self) -> Any:
        return get_current_user_info(self)

    def get_stamp_templates(self, project_uuid: str, page: int = 0) -> Any:
        return get_stamp_templates(self, project_uuid, page)

    def get_user_reports(
        self, license_uuid: str, limit: int = 100, page: int = 0
    ) -> Any:
        return get_user_reports(self, license_uuid, limit, page)

    def get_project_sheets(self, project_uuid: str) -> Any:
        return get_project_sheets(self, project_uuid)

    def get_sheet_history(self, project_uuid: str, sheet_uuid: str) -> Any:
        return get_sheet_history(self, project_uuid, sheet_uuid)

    def get_sheet_filter_options(self, project_uuid: str) -> Any:
        return get_sheet_filter_options(self, project_uuid)

    def get_deleted_issues(
        self,
        project_uuid: str,
        page: int = 0,
        limit: int = 100,
        additional_fields: Optional[List[str]] = None,
        always_filters_dto: Optional[List[Dict[str, Any]]] = None,
        any_filters_dto: Optional[List[Dict[str, Any]]] = None,
        report_sort: Optional[List[str]] = None,
        send_full_issue_data: bool = False,
        statuses: Optional[List[str]] = None,
        synchronized: Optional[str] = None,
    ) -> Any:
        return get_deleted_issues(
            self,
            project_uuid,
            page,
            limit,
            additional_fields,
            always_filters_dto,
            any_filters_dto,
            report_sort,
            send_full_issue_data,
            statuses,
            synchronized,
        )

    def create_issue(
        self,
        preview_file_path: str,
        project_id: int,
        fields_json: Dict[str, Any],
        clash_test_uuid: Optional[str] = None,
        operation_id: Optional[str] = None,
    ) -> Any:
        return create_issue(
            self,
            preview_file_path,
            project_id,
            fields_json,
            clash_test_uuid,
            operation_id,
        )


import requests

def get_project_roles(auth_client, license_uuid):
    """
    Get the list of project roles from a license.

    :param license_uuid: The UUID of the license.
    :param token: The bearer token for authorization.
    :return: A JSON object with the response data.
    """
    
    url = f"https://api.{auth_client.region}.revizto.com/v5/license/{license_uuid}/role/list"
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Accept': 'application/json'
    }

    response = requests.get(url, headers=headers, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(data)

    return data

def assign_project_role(auth_client, project_uuid, member_uuids, role_uuid, operation_id=None):
    """
    Assign a project role to one or several project members.

    :param project_uuid: The UUID of the project.
    :param token: The bearer token for authorization.
    :param member_uuids: A list of UUIDs of project members.
    :param role_uuid: The UUID of the project role to assign.
    :param operation_id: An optional unique string associated with notifications.
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/project/{project_uuid}/role/bulk-edit"
    
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Accept': 'application/json',
        'Content-Type': 'application/json'
    }
    
    payload = {
        'memberUuids': member_uuids,
        'roleUuid': role_uuid
    }
    
    if operation_id is not None:
        payload['operationId'] = operation_id

    response = requests.post(url, headers=headers, json=payload, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(data)

    return data



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()


VERSION = '0.91'
DESCRIPTION = 'pyRevizto is a library designed specifically for Python developers who work with Revizto, a popular platform for BIM collaboration and issue tracking. By leveraging pyRevizto, developers can streamline their workflows and significantly enhance the capabilities of Revizto through automation.'

# Setting up
setup(
    name="pyrevizto",
    version=VERSION,
    author="Umar Khalid",
    description=DESCRIPTION,
    packages=find_packages(),
    install_requires=['requests>=2.28.2', 
                      'python-dotenv>=1.0.1'],
    keywords=['Revizto'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    python_requires='>=3.6',
    url="https://github.com/umarkhalid007/pyRevizto",
    project_urls={
        "Bug Tracker": "https://github.com/umarkhalid007/pyRevizto/issues",
    },
)


import requests

def get_current_user_licenses(auth_client):
    """Fetch the current user's licenses using the provided pyRevizto instance."""
    if not auth_client.access_token:
        raise ValueError("Access token is not available.")

    url = f"https://api.{auth_client.region}.revizto.com/v5/user/licenses"
    
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Content-Type': 'application/json'
    }

    response = requests.get(url, headers=headers, verify=auth_client.verification_bool, cert=auth_client.certificate)

    if response.status_code == 401:  # Token expired
        auth_client.refresh_tokens()
        headers['Authorization'] = f"Bearer {auth_client.access_token}"
        response = requests.get(url, headers=headers)
    if response.status_code == 200:
        return response.json()
    else:
        return response.text
    
def get_license_members(auth_client, license_id):
    if not auth_client.access_token:
        raise ValueError("Access token is not available.")

    url = f"https://api.{auth_client.region}.revizto.com/v5/license/{license_id}/team"

    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Content-Type': 'application/json'
    }

    response = requests.get(url, headers=headers, verify=auth_client.verification_bool, cert=auth_client.certificate)

    if response.status_code == 200:
        if response.json().get("result") == -206:  # Token expired
            auth_client.get_refreshed_token()
            headers['Authorization'] = f"Bearer {auth_client.access_token}"
            response = requests.get(url, headers=headers)

        return response.json()
    else:
        return response.text
        
def invite_users_to_license(auth_client, license_uuid, invite_data, preserve_roles=None, 
                            make_guests=None, auth_method=None, deactivate=None, operation_id=None):
    """
    Invite users to a license.

    :param license_uuid: The UUID of the license.
    :param token: The bearer token for authorization.
    :param invite_data: A list of dictionaries, where each dictionary contains user information.
    :param preserve_roles: Whether to preserve current license roles for existing users (boolean).
    :param make_guests: Whether to grant 'Guest' role to eligible users (boolean).
    :param auth_method: The UUID of the authentication method to set for new users (string).
    :param deactivate: Whether to deactivate the users (boolean).
    :param operation_id: A unique string associated with notifications (string).
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/license/{license_uuid}/invite/bulk"
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Content-Type': 'application/json'
    }
    #invite_data should be an array
    payload = {
        'data': invite_data
    }
    
    if preserve_roles is not None:
        payload['preserveRoles'] = preserve_roles
    if make_guests is not None:
        payload['makeGuests'] = make_guests
    if auth_method is not None:
        payload['authMethod'] = auth_method
    if deactivate is not None:
        payload['deactivate'] = deactivate
    if operation_id is not None:
        payload['operationId'] = operation_id

    response = requests.post(url, headers=headers, json=payload, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(f"API Error: {data['message']}")

    return data

def assign_license_roles(auth_client, license_uuid, member_uuids, role, operation_id=None):
    """
    Assign a license role to license members.

    :param license_uuid: The UUID of the license.
    :param token: The bearer token for authorization.
    :param member_uuids: A list of UUIDs of license members.
    :param role: The license role to assign (integer: 1-Guest, 2-Collaborator, 3-Content creator, 4-License administrator).
    :param operation_id: A unique string associated with notifications (optional).
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/license/{license_uuid}/edit/role/bulk"
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Accept': 'application/json',
        'Content-Type': 'application/json'
    }
    
    payload = {
        'uuids': member_uuids,
        'role': role,
    }
    
    if operation_id is not None:
        payload['operationId'] = operation_id

    response = requests.post(url, headers=headers, json=payload, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(f"API Error: {data['message']}")

    return data

def remove_license_members(auth_client, license_uuid, member_uuids, message=None, operation_id=None):
    """
    Remove license members from a license.

    :param license_uuid: The UUID of the license.
    :param token: The bearer token for authorization.
    :param member_uuids: A list of UUIDs of license members to be removed.
    :param message: The message to be sent to the users removed from the license (optional).
    :param operation_id: A unique string associated with notifications (optional).
    :return: A JSON object with the response data.
    """
    url = f"https://api.{auth_client.region}.revizto.com/v5/license/{license_uuid}/remove-member/bulk"
    headers = {
        'Authorization': f'Bearer {auth_client.access_token}',
        'Accept': 'application/json',
        'Content-Type': 'application/json'
    }
    
    payload = {
        'uuids': member_uuids
    }
    
    if message is not None:
        payload['message'] = message
    if operation_id is not None:
        payload['operationId'] = operation_id

    response = requests.post(url, headers=headers, json=payload, verify=auth_client.verification_bool, cert=auth_client.certificate)
    
    if response.status_code != 200:
        response.raise_for_status()

    data = response.json()
    
    if data['result'] != 0:
        raise Exception(f"API Error: {data['message']}")

    return data
